package jbpmExplained;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.event.process.ProcessEventListener;
import org.drools.io.impl.ClassPathResource;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessInstance;

public class JbpmExplained {

	public static void main(String[] args) throws IOException {
		new JbpmExplained();
	}

	public JbpmExplained() throws IOException {
		setup();
		startTheProcess();
	}

	// The file names of your processes
	private final String fileName1 = "ExplanationProcess1.bpmn";
	private final String fileName2 = "ExplanationProcess2.bpmn";
	// The process ids of your processes (can be set in the bpmn file. Open the
	// bpmn file with BPMN Editor and open the properties view (Window, show
	// view, general, properties)
	private final String processID1 = "ExplanationProcess1";
	private final String processID2 = "ExplanationProcess2";
	// A session can start processes, set and get globals and listens to
	// (process-)
	// events. Furthermore it interacts with the KnowledgeBase.
	private StatefulKnowledgeSession ksession;
	// This listener reacts on several events that can occur during a process
	// such as beforeProcessStarted, afterNodeTriggered,afterNodeLeft etc.
	private ProcessEventListener processEventListener;
	// A TaskHandler is needed in order to define what has to be done when a
	// WorkItem is being executed or aborted. You can also provide a
	// completeWorkItem() method in your TaskHandler to change the behaviour of
	// a workItem when it gets completed.
	MyTaskHandler taskHandler;
	// A ProcessInstance is an instance of a process and contains all runtime
	// information related to that instance. A process defined by a
	// BPMN file can be instantiated multiple times.
	ProcessInstance process1;
	ProcessInstance process2;

	public void setup() throws IOException {
		this.ksession = createKSession();
		// Adds the eventListener to the session TODO
		ksession.addEventListener(processEventListener);
		this.taskHandler = new MyTaskHandler();
		// Every session has a WorkItemManager which is responsible for finding
		// the right WorkItemHandler for a WorkItem. So we need to
		// register our taskHandler to our WorkItem (which must have a unique
		// name). In this case, it is a custom WorkItem with the name
		// "MeyleTask"
		this.ksession.getWorkItemManager().registerWorkItemHandler("MyTask", taskHandler);
		MySingleton.getInstance().setKsession(ksession);
	}

	public void startTheProcess() {
		// It is possible if needed to pass some Parameters to the process that
		// can be accessed from WorkItems during the execution of a process.
		Map<String, Object> someParametersForTheProcess = new HashMap<String, Object>();
		String aStringNeededBySomeNodesInProcess = "I am just a string";
		someParametersForTheProcess.put("objectForProcess", aStringNeededBySomeNodesInProcess);
		// Starts the Process with id processID1
		process1 = ksession.startProcess(processID1, someParametersForTheProcess);
		// If no parameters are needed you can start a process without
		// parameters.
		process2 = ksession.startProcess(processID2);		
	}

	private StatefulKnowledgeSession createKSession() {
		// Creates a KnowledgeBuilder which is used to build the KnowledgeBase
		// definitions that are held collectively in KnowledgePackages.
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();

		// Adds a process definition of ResourceType BPMN2 to the
		// KnowledgeBuilder. (there are several ResourceTypes such as drl files
		// xls etc.) fileName is the file name of the BPMN file.
		// (In the end the KnowledgeBuilder will add the ressource to the
		// package of the KnowledgeBase, see @Info some lines below)
		kbuilder.add(new ClassPathResource(fileName1), ResourceType.BPMN2);
		kbuilder.add(new ClassPathResource(fileName2), ResourceType.BPMN2);

		// This is a somewhat standard procedure one should do in order to be
		// sure that no errors occured during the creation of the
		// KnowledgeBuilder.
		// Quote from KnowledgeBuilder:
		// "It is best practice to always check the hasErrors() method after an
		// addition, you should not add more resources or get the
		// KnowledgePackages if there are errors. KnowledgePackages() will
		// return an empty list if there are errors."
		if (kbuilder.hasErrors()) {
			if (kbuilder.getErrors().size() > 0) {
				for (KnowledgeBuilderError error : kbuilder.getErrors()) {
					System.out.println("Error building kbase: " + error.getMessage());
				}
			}
			throw new RuntimeException("Error building kbase!");
		}

		@Info
		// Creates a KnowledgeBase and adds the generated package. The name of
		// the package is set in the *.bpmn process file. Property package
		// determines the package name.
		KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
		kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());

		// Creates and returns a new statefull session.
		return kbase.newStatefulKnowledgeSession();
	}
}